/* main.cpp */
/* Copyright (C) 2015 nimbelink.com, MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "mbed.h" 
#include "LPS331.h"
#include "LIS3DH.h"
#include "LM75B.h"
#include "DS1820.h"

// we assign teh DeviceID for the 4G LTE device using the modem's IMEI

 /* --CHANGE THIS FOR YOUR SETUP" -- */
 //#define DeviceID "353238060170445"  //Freeboard DweetIO unique ID

const int MAX_PROBES = 3;
DS1820 probe[3] = {D3, D3, D3};



DigitalOut myled(LED1);
DigitalOut skywire_en(PA_6);
DigitalOut skywire_rts(PA_7);

Serial skywire(PA_9,PA_10);
Serial debug_pc(USBTX, USBRX);

I2C i2c(PB_9,PB_8);
char msg[3];

LPS331 pressure(i2c);
LM75B LM75_temp(PB_9,PB_8);
LIS3DH accel(i2c, LIS3DH_V_CHIP_ADDR, LIS3DH_DR_NR_LP_100HZ, LIS3DH_FS_2G);


char str[255];
char DeviceID[255];

float latitude;
float longitude;
int number;
int retry;

volatile int rx_in=0;
volatile int rx_out=0;
const int buffer_size = 255;
char rx_buffer[buffer_size+1];

char rx_line[buffer_size];

void Skywire_Rx_interrupt() {
// Loop just in case more than one character is in UART's receive FIFO buffer
// Stop if buffer full
    while ((skywire.readable()) && (((rx_in + 1) % buffer_size) != rx_out)) {
        rx_buffer[rx_in] = skywire.getc();
        rx_in = (rx_in + 1) % buffer_size;
    }
    return;
}

void read_line() {
    int i;
    i = 0;
// Start Critical Section - don't interrupt while changing global buffer variables
    __disable_irq();
// Loop reading rx buffer characters until end of line character
    while ((i==0) || (rx_line[i-1] != '\n')) {
// Wait if buffer empty
        if (rx_in == rx_out) {
// End Critical Section - need to allow rx interrupt to get new characters for buffer
            __enable_irq();
            while (rx_in == rx_out) {
            }
// Start Critical Section - don't interrupt while changing global buffer variables
            __disable_irq();
        }
        rx_line[i] = rx_buffer[rx_out];
        i++;
        rx_out = (rx_out + 1) % buffer_size;
    }
// End Critical Section
    __enable_irq();
    rx_line[i-1] = 0;
    return;
}
int WaitForResponse(char* response1, char* response2, int num) {
    do {
        read_line();
        debug_pc.printf("Waiting for: %s or %s, Recieved: %s\r\n", response1, response2, rx_line);
    } while (strncmp(rx_line, response1, num) && strncmp(rx_line, response2, num));
    if (!strncmp(rx_line, response1, num)) {
                return 0;
            }else {
                return 1;
                }
}



int main()
{
    float axis[3];
    float press;
    float temp;
    int i;
    int devices_found=0;

    debug_pc.baud(115200);
    skywire.baud(115200);
    debug_pc.printf("SystemCoreClock = %d Hz\n", SystemCoreClock);
    
    skywire.attach(&Skywire_Rx_interrupt, Serial::RxIrq);
    skywire_rts=0;
    myled=0;
    debug_pc.printf("Starting Demo...\r\n");
    debug_pc.printf("Waiting for Skywire to Boot...\r\n");
    //Enable Skywire
    skywire_en=0;
    wait(4);
    skywire_en=1;
    wait(4);
    skywire_en=0;

    myled=1;
    wait(5);

    LM75_temp.open();
    
    //Turn off echo
    skywire.printf("ATE0\r\n");
    WaitForResponse("OK","ER", 2);
    debug_pc.printf("Received ATE0 OK...\r\n");  
    
    // read out 4G LTE IMEI number to use as unique identifier
    skywire.printf("AT+GSN\r\n");
    wait(2);
    read_line(); 
    read_line(); 
    sscanf(rx_line, "%15s,%*s,%*s", DeviceID);

    debug_pc.printf("Device IMEI: %s \r\n", DeviceID ); 
    //DeviceID = str;
    // required setup PDP context for 4G LTE connections
    skywire.printf("AT+CGDCONT=3,\"IP\",\"VZWINTERNET\",\"\",0,0\r\n");  
    WaitForResponse("OK","ER", 2);
    
    wait(10);
    
    /* skywire.printf("AT+CGDCONT?\r\n");  
        
    sscanf(rx_line, "%s", str);
    debug_pc.printf("Debug AT+CGDCONT? response: %s", str);  
    */
    
    //skywire.printf("AT+CGREG\r\n");  
    //WaitForResponse("0,1", 2);   
    
    // required setup socket for 4G LTE connections
    skywire.printf("AT#SCFG=3,3,300,90,600,50\r\n");  
    WaitForResponse("OK","ER", 2);
    
    
    debug_pc.printf("Connecting to Network...\r\n");
    wait(1);
    
    // loop until we get an IP address
    retry = 1;
    while (retry == 1){
        debug_pc.printf("Sending AT#SGACT command...\r\n");   
        skywire.printf("AT#SGACT=3,1\r\n");
        //retry = WaitForResponse("OK","ER", 2);
        //WaitForResponse("#SGAC","ERROR", 5);
   //     if (retry == 0){
               retry = WaitForResponse("#SGAC","ERROR", 5);
                    
                    if (retry == 0){
                        debug_pc.printf("Received IP address OK...\r\n");      
                        WaitForResponse("OK","ER", 2);
                        } else{
                            debug_pc.printf("Sending socket close and waiting 5 seconds...\r\n");                        
                            skywire.printf("AT#SGACT=3,0\r\n");
                            WaitForResponse("OK","ER", 2);
                            wait(5);
                        }
     
        //}else{
          //  wait(2);    
        //}
    }
     
     
     
    

 
 
 
 // find probes
    debug_pc.printf("search_ROM_setup\r\n");
    probe[0].search_ROM_setup();
    debug_pc.printf("search_ROM\r\n");
    while (probe[devices_found].search_ROM() and devices_found<MAX_PROBES-1)
          devices_found++;
    
    // If maximum number of probes are found,
    // bump the counter to include the last array entry
    if (probe[devices_found].ROM[0] != 0xFF)
        devices_found++;
    debug_pc.printf("devices found:%d\r\n", devices_found);


    //while(1) {
        probe[0].convert_temperature(DS1820::all_devices);
        for (i=0; i<devices_found; i++) {
            debug_pc.printf("Device[%d]: %3.1f \r\n",i, probe[i].temperature('c'));
        }
    //    wait(1.0); // 1 sec
    //}
 
 
    wait(3);
    debug_pc.printf("Reading sensors...\r\n");      
    while(1) {
        temp = (float)LM75_temp;
        debug_pc.printf("Temp = %.3f\r\n", temp);
        press=(float)pressure.value() / 4096;
        debug_pc.printf("Pressure = %.3f\r\n", press);
        accel.read_data(axis);
        debug_pc.printf("Accel = %.3f, %.3f, %.3f\r\n", axis[0], axis[1], axis[2]);

        probe[0].convert_temperature(DS1820::all_devices);
        for (i=0; i<devices_found; i++) {
            debug_pc.printf("Device[%d]: %3.1f \r\n",i, probe[i].temperature('c'));
        }
        
        
        // alternative code that supports 4G LTE modem
        debug_pc.printf("Initiating socket dial...\r\n");
        

        skywire.printf("at#sd=3,0,80,\"dweet.io\"\r\n");
        retry = WaitForResponse("CONNECT","NO CARR", 7);
        if (retry == 0 ){
            skywire.printf("POST /dweet/for/%s?temperature=%.3f&pressure=%.3f&T=%3.1f&H=%3.1f&O=%3.1f HTTP/1.1", DeviceID, temp, press, probe[0].temperature('f'), probe[1].temperature('f'), probe[2].temperature('f'));
            skywire.putc(0x0A);
            skywire.putc(0x0A); 
            
             //WaitForResponse("HTTP/1.1 2","NO CARRIER", 10);
   
            retry = WaitForResponse("HTTP/1.1 2","NO CARRIER", 10);
            if (retry == 0){ 
                skywire.printf("+++");
                WaitForResponse("NO CARRIER","NO CARRIER", 10);
                debug_pc.printf("Socket closed...\r\n");
            }
        }
                skywire.printf("AT#SH=3\r\n");
                WaitForResponse("OK","ER", 2);
        debug_pc.printf("Waiting to take next reading...\r\n");
        wait(10);
    }

}
